@@ -11,16 +11,18 @@ VersionInfo = NamedTuple(
     "VersionInfo", major=int, minor=int, micro=int, releaselevel=str, serial=int
 )
 
-version_info = VersionInfo(major=0, minor=2, micro=1, releaselevel="", serial=0)
+version_info = VersionInfo(major=0, minor=3, micro=0, releaselevel="", serial=0)
 
 __title__ = "baguette"
 __author__ = "takos22"
-__version__ = "0.2.1"
+__version__ = "0.3.0"
 
 __all__ = [
     "Baguette",
+    "Config",
     "Headers",
     "make_headers",
+    "Middleware",
     "render",
     "Response",
     "HTMLResponse",
@@ -33,12 +35,14 @@ __all__ = [
     "make_error_response",
     "redirect",
     "Request",
+    "TestClient",
     "View",
 ]
 
-from . import testing
 from .app import Baguette
+from .config import Config
 from .headers import Headers, make_headers
+from .middleware import Middleware
 from .rendering import render
 from .request import Request
 from .responses import (
@@ -53,4 +57,5 @@ from .responses import (
     make_response,
     redirect,
 )
+from .testing import TestClient
 from .view import View
