@@ -27,8 +27,8 @@ class Headers:
     def get(self, name, default=None):
         """Gets a header from its name. If not found, returns ``default``.
 
-        Parameters
-        ----------
+        Arguments
+        ---------
             name: :class:`str` or :class:`bytes`
                 Name of the header to get.
             default: Optional anything
@@ -111,7 +111,9 @@ class Headers:
         del self._headers[name.lower()]
 
     def __contains__(self, name):
-        return name in self._headers
+        if isinstance(name, bytes):
+            name = name.decode("ascii")
+        return name.lower() in self._headers
 
     def __add__(self, other: typing.Union["Headers", typing.Mapping[str, str]]):
         new = Headers(**self)
@@ -134,8 +136,8 @@ def make_headers(headers: HeadersType = None) -> Headers:
     """Makes a :class:`Headers` object from a :class:`list` of
         ``(str, str)`` tuples, a :class:`dict`, or a :class:`Headers` instance.
 
-        Parameters
-        ----------
+        Arguments
+        ---------
             headers: :class:`list` of ``(str, str)`` tuples, \
             :class:`dict` or :class:`Headers`
                 The raw headers to convert.
